library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- The interface to the outside world
entity stopwatch is
	port (clk : in std_logic;
	      areset : in std_logic;
			start : in std_logic;
			clear : in std_logic;
			ready : out std_logic;
			hex3 : out std_logic_vector(6 downto 0);
			hex2 : out std_logic_vector(6 downto 0);
			hex1 : out std_logic_vector(6 downto 0);
			hex0 : out std_logic_vector(6 downto 0);
			pulse : out std_logic
		  );
end entity stopwatch;

architecture structural of stopwatch is
-- Declaration of the reset synchonizer
component reset_synchronizer is
    port (clk: in std_logic;
          areset: in std_logic;
          areset_sync: out std_logic
         );
end component reset_synchronizer;
-- Declaration of the synchonizer chains
component synchronizer_chain is
	port (clk : in std_logic;
			areset : in std_logic;
			d : in std_logic;
			q: out std_logic
		  );
end component synchronizer_chain;
-- Declaration of the control
component control is
	port (clk : in std_logic;
			areset : in std_logic;
			start : in std_logic;
			clear : in std_logic;
			ready : out std_logic;
			countup : out std_logic;
			clearout : out std_logic
		  );
end component control;
-- Declaration of the counter
component counter is
	generic (sysfreq : integer := 50000000;
	         watchfreq : integer := 100
	        );
	port (clk : in std_logic;
	      areset : in std_logic;
			clear : in std_logic;
			countup : in std_logic;
			tensec : out unsigned(3 downto 0);
			sec : out unsigned(3 downto 0);
			secten : out unsigned(3 downto 0);
			sechund : out unsigned(3 downto 0);
			pulse : out std_logic
		  );
end component counter;
-- Declaration of the seven segment deooders
component seven_segment is
    port (data   : in unsigned(3 downto 0);
          leds  : out std_logic_vector(6 downto 0)
         );
end component seven_segment;
signal countup_int : std_logic;
signal clearout_int : std_logic;
signal tensec_int : unsigned(3 downto 0);
signal sec_int : unsigned(3 downto 0);
signal secten_int : unsigned(3 downto 0);
signal sechund_int : unsigned(3 downto 0);
signal start_int : std_logic;
signal clear_int : std_logic;
signal pulse_int : std_logic;
signal hex3_int : std_logic_vector(6 downto 0);
signal hex2_int : std_logic_vector(6 downto 0);
signal hex1_int : std_logic_vector(6 downto 0);
signal hex0_int : std_logic_vector(6 downto 0);
signal start_s : std_logic;
signal clear_s : std_logic;
signal areset_s : std_logic;

begin

	-- Instantiate reset synchronizer
	u0: reset_synchronizer
	port map (clk => clk, areset => areset, areset_sync => areset_s);

	-- Instantiate the synchronizer chains
	u1: synchronizer_chain
	port map (clk => clk, areset => areset_s, d => start, q => start_s);
	
	u2: synchronizer_chain
	port map (clk => clk, areset => areset_s, d => clear, q => clear_s);
	
   -- start and clear inputs are active low
	start_int <= not start_s;
	clear_int <= not clear_s;

	-- instantiate control block
	u3: control
	port map (clk => clk, areset => areset_s, start => start_int,
	          clear => clear_int, ready => ready, countup => countup_int,
				 clearout => clearout_int);

	-- instantiate counter block
	u4: counter
	generic map (sysfreq => 50000000, watchfreq => 100)
	port map (clk => clk, areset => areset_s, clear => clearout_int,
             countup => countup_int, tensec => tensec_int,
				 sec => sec_int, secten => secten_int,
				 sechund => sechund_int, pulse => pulse_int);

	-- instantiate four seven segment decoder/drivers
	u5: seven_segment
	port map (data => sechund_int, leds => hex0_int);

	u6: seven_segment
	port map (data => secten_int, leds => hex1_int);

	u7: seven_segment
	port map (data => sec_int, leds => hex2_int);

	u8: seven_segment
	port map (data => tensec_int, leds => hex3_int);

	-- pulse output is active low
	pulse <= not pulse_int;
	
	-- seven segment display are active low
	hex0 <= not hex0_int;
	hex1 <= not hex1_int;
	hex2 <= not hex2_int;
	hex3 <= not hex3_int;
	
end architecture structural;
